'use strict';

import path from 'path';
import Knex from 'knex';
import Quture from 'quture-app';
import Config from 'quture-config';

export default class Database {

  /**
   * Create a new {Database} instance.
   *
   * @param {Quture} quture
   * @param {string} migrationCommand
   */
  constructor(quture) {
    const self = this;

    this._quture = quture;
    this._config = new Config({
      configDir: path.join(this._quture.getConfig().getConfigDirectory(), 'database')
    });

    /**
     * Publish configuration file
     */
    this._quture.publishConfigurationFile(this._config, path.join(__dirname, 'config', 'connection.js'));

    /**
     * Configure knex
     */
    this._migrationsDir = this._config.get('connection.migrations', path.join(this._quture.getRootDirectory(), 'migrations'));

    let options = {
      client: this._config.get('connection.dialect', 'sqlite'),
      connection: {
        host: this._config.get('connection.host', '127.0.0.1'),
        user: this._config.get('connection.username', 'root'),
        password: this._config.get('connection.password', ''),
        database: this._config.get('connection.database', 'quture'),
        filename: this._config.get('connection.storage', path.join(this._quture.getStorageDirectory(), 'database.sqlite'))
      },
      migrations: {
        directory: this._migrationsDir,
        tableName: 'migrations'
      }
    };

    /**
     * Instantiate knex
     */
    this._knex = Knex(options);

    /**
     * Setup getQutureDatabase() method for allowing
     * the ability to retrieve this instance from
     * within migrations.
     */
    this._knex.getQutureDatabase = function() {
      return this;
    }
  }

  /**
   * Retrieve the {Knex} instance.
   *
   * @returns {Knex}
   */
  knex() {
    return this._knex;
  }

  /**
   * Retrieve the migrations directory path.
   *
   * @return {string}
   */
  getMigrationsDirectory() {
    return this._migrationsDir;
  }

  /**
   * Check if the migration file with the specified name
   * is already published to the migrations folder.
   *
   * @param {string} fileName
   * @returns {boolean}
   */
  isMigrationPublished(fileName) {
    return Quture.getUtilities().isFilePublished(fileName, this.getMigrationsDirectory());
  }

  /**
   * Publish the specified migration file into the
   * migration folder.
   *
   * @param {string} filePath
   */
  publishMigration(filePath) {
    return Quture.getUtilities().publishFile(filePath, this.getMigrationsDirectory());
  }

}
